##############################################################################
#
# SubPlone - Simple, nested sites to go
# Copyright (C) 2005 Sidnei da Silva and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: skin.py 8663 2005-05-23 16:32:15Z dreamcatcher $
"""

from Globals import Persistent
from ExtensionClass import Base
from Acquisition import Implicit, aq_inner, aq_parent, aq_base
from OFS.Traversable import Traversable
from ZPublisher.BeforeTraverse import registerBeforeTraverse
from ZPublisher.BeforeTraverse import unregisterBeforeTraverse
from Products.SiteAccess.AccessRule import AccessRule
from Products.SubPlone.config import *

class TrueCondition:

    def __call__(self, container, request):
        return True

class SkinSwitcher(Base):

    def __call__(self, container, request):
        if container.skin_name is None:
            return
        if not container.condition(container, request):
            return
        container.changeSkin(container.skin_name)

HOOK_NAME = 'skin_switcher'
PRIORITY = 2
FLAG = '_skin_traverser_installed'

class Skin(Persistent, Traversable, Implicit):
    """ Allow to choose current skin based on a condition
    """

    skin_name = None
    condition = TrueCondition()
    skin_switcher = SkinSwitcher()

    def updateTraversalData(self, adding=False):
        if hasattr(aq_base(self), FLAG):
            unregisterBeforeTraverse(self, HOOK_NAME)
            del self._skin_traverser_installed

        if adding:
            hook = AccessRule(HOOK_NAME)
            registerBeforeTraverse(self, hook, HOOK_NAME, PRIORITY)
            setattr(self, FLAG, True)

    def manage_afterAdd(self, item, container):
        """ Add our traversal hook
        """
        self.updateTraversalData(True)

    def manage_beforeDelete(self, item, container):
        """ Clean up on delete
        """
        self.updateTraversalData(False)
